function [alpha_Combined,MFS_Combined] = mfsAll_kmean(IM,varargin) 
%mfsAll Computes combined  MFS measurements consisting  of three kinds: Intensity, Gradients, Laplacian.
%
%mfsAll(IM) computes the MFS vectors for three kinds of measurements
%   1)  Intensity
%   2)  Gradients
%   3)  Laplacian
%
%[alpha_Combined,MFS_Combined] = mfsAll(IM) returns the vector including three types of MFS
%                          for input image IM
%
%
%Author: Yong Xu, Hui Ji
%Date: Apr 24, 2007
%



switch nargin,
    
    case 2,
        ind_num = varargin{1};  %density counting levels
        f_num = 26;  %the dimension of MFS
        ite_num = 3; % iteration levels in estimating fractal dimension
    case 3,
        ind_num = varargin{1};
        f_num = varargin{2};
        ite_num = 3;
    case 4,    
        ind_num = varargin{1};
        f_num = varargin{2};
        ite_num = varargin{3};
    otherwise,
        ind_num = 1; %density counting levels
        f_num = 26;  %the dimension of MFS
        ite_num = 3; %Box counting levels  
end;
% Preprocessing 
if (length(size(IM)) == 3),
    IM = rgb2gray(IM);
end;
IM = double(IM);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following code for W, WG, WL should 
% be replaced by the quantity from training set
%Compute Intensity
W = sum(double(IM(:)));

%Compute Gradient

fx = 1/2*[-1 0 1];
fy = fx';
fxy = 1/2*[-1 0 0;0 0 0;0 0 1];
fyx = 1/2*[0 0 -1;0 0 0;1 0 0]; 

IMG = single(IM);
IMG = (conv2(IMG,fx,'same')).^2 + (conv2(IMG,fy,'same')).^2+(conv2(IMG,fxy,'same')).^2 +(conv2(IMG,fyx,'same')).^2;
IMG = sqrt(IMG);
IMG = uint8(fix(IMG)); 

WG = sum(double(IMG(:)));

%Compute Laplacian

f1 = fspecial('gaussian',5,1); 
f2 = [-1 -1 -1;-1 8 -1;-1 -1 -1];
f = conv2(f1,f2);  

IML = single(IM);
IML = conv2(single(IML),f,'same');
IML = uint8(fix(IML)); 

WL = sum(double(IML(:)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% MFS of Intensity
im = double(IM);
im = im * W /sum(im(:));
[a_m,m] = mfs_kmean(im,ind_num,f_num,ite_num);

%MFS of Gradient

imG = single(IM);
imG = (conv2(imG,fx,'same')).^2 + (conv2(imG,fy,'same')).^2 + (conv2(imG,fxy,'same')).^2 + (conv2(imG,fyx,'same')).^2;
imG = sqrt(imG);
imG = uint8(fix(imG)); 

imG = double(imG);
imG = imG * WG /sum(imG(:));
[a_mG,mG] = mfs_kmean(imG,ind_num,f_num, ite_num);

%MFS of Laplacion

imL = single(im);
imL = conv2(single(imL),f,'same');
imL = uint8(fix(imL)); 

imL = double(imL);
imL = imL * WL /sum(imL(:));
[a_mL,mL] = mfs_kmean(imL,ind_num,f_num, ite_num);

%Comine all three vector 
    alpha_Combined{1} = a_m;
    alpha_Combined{2} = a_mG;
    alpha_Combined{3} = a_mL;
    MFS_Combined{1} = m;
    MFS_Combined{2} = mG;
    MFS_Combined{3} = mL;
return;

